/*
 * @adonisjs/http-server
 *
 * (c) AdonisJS
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */
import * as plainCookiesDriver from './drivers/plain.js';
import * as signedCookiesDriver from './drivers/signed.js';
import * as encryptedCookiesDriver from './drivers/encrypted.js';
/**
 * Cookie client exposes the API to parse/set AdonisJS cookies
 * as a client.
 */
export class CookieClient {
    #encryption;
    constructor(encryption) {
        this.#encryption = encryption;
    }
    /**
     * Encrypt a key value pair to be sent in the cookie header
     */
    encrypt(key, value) {
        return encryptedCookiesDriver.pack(key, value, this.#encryption);
    }
    /**
     * Sign a key value pair to be sent in the cookie header
     */
    sign(key, value) {
        return signedCookiesDriver.pack(key, value, this.#encryption);
    }
    /**
     * Encode a key value pair to be sent in the cookie header
     */
    encode(_, value) {
        return plainCookiesDriver.pack(value);
    }
    /**
     * Unsign a signed cookie value
     */
    unsign(key, value) {
        return signedCookiesDriver.canUnpack(value)
            ? signedCookiesDriver.unpack(key, value, this.#encryption)
            : null;
    }
    /**
     * Decrypt an encrypted cookie value
     */
    decrypt(key, value) {
        return encryptedCookiesDriver.canUnpack(value)
            ? encryptedCookiesDriver.unpack(key, value, this.#encryption)
            : null;
    }
    /**
     * Decode an encoded cookie value
     */
    decode(_, value) {
        return plainCookiesDriver.canUnpack(value) ? plainCookiesDriver.unpack(value) : null;
    }
    /**
     * Parse response cookie
     */
    parse(key, value) {
        /**
         * Unsign signed cookie
         */
        if (signedCookiesDriver.canUnpack(value)) {
            return signedCookiesDriver.unpack(key, value, this.#encryption);
        }
        /**
         * Decrypted encrypted cookie
         */
        if (encryptedCookiesDriver.canUnpack(value)) {
            return encryptedCookiesDriver.unpack(key, value, this.#encryption);
        }
        /**
         * Decode encoded cookie
         */
        if (plainCookiesDriver.canUnpack(value)) {
            return plainCookiesDriver.unpack(value);
        }
    }
}
