/*
 * @adonisjs/http-server
 *
 * (c) AdonisJS
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */
import { RuntimeException } from '@poppinss/utils';
import { parseRoutePattern } from '../parser.js';
/**
 * URL builder class is used to create URIs for pre-registered
 * routes.
 *
 * ```ts
 * const builder = new UrlBuilder(encryption, routeFinder)
 *
 * builder
 *  .qs({ sort: 'id' })
 *  .params([category.id])
 *  .make('categories.posts.index')
 * ```
 */
export class UrlBuilder {
    /**
     * Query string parser
     */
    #qsParser;
    /**
     * The parameters to apply on the route
     */
    #params = {};
    /**
     * Query string to append to the route
     */
    #qs = {};
    /**
     * Should we perform the route lookup or just build the
     * given pattern as it is.
     */
    #shouldPerformLookup = true;
    /**
     * BaseURL to append to the constructored URL
     */
    #baseUrl;
    /**
     * Encryption class for making signed URLs
     */
    #encryption;
    /**
     * Route finder for finding route pattern
     */
    #routeFinder;
    constructor(encryption, routeFinder, qsParser) {
        this.#qsParser = qsParser;
        this.#encryption = encryption;
        this.#routeFinder = routeFinder;
    }
    /**
     * Raises exception when wildcard values array is missing or
     * has length of zero.
     */
    #ensureHasWildCardValues(pattern, values) {
        if (!values || !Array.isArray(values) || !values.length) {
            throw new RuntimeException(`Cannot make URL for "${pattern}" route. Invalid value provided for wildcard param`);
        }
    }
    /*
     * Raises exception when value is not defined
     */
    #ensureHasParamValue(pattern, param, value) {
        if (value === undefined || value === null) {
            throw new RuntimeException(`Cannot make URL for "${pattern}" route. Missing value for "${param}" param`);
        }
    }
    /**
     * Processes the pattern against the params
     */
    #processPattern(pattern) {
        const uriSegments = [];
        const paramsArray = Array.isArray(this.#params) ? this.#params : null;
        const paramsObject = !Array.isArray(this.#params) ? this.#params : {};
        let paramsIndex = 0;
        const tokens = parseRoutePattern(pattern);
        for (const token of tokens) {
            /**
             * Expected wildcard param to be at the end always and hence
             * we must break out from the loop
             */
            if (token.type === 0) {
                uriSegments.push(`${token.val}${token.end}`);
            }
            else if (token.type === 2) {
                const values = paramsArray ? paramsArray.slice(paramsIndex) : paramsObject['*'];
                this.#ensureHasWildCardValues(pattern, values);
                uriSegments.push(`${values.join('/')}${token.end}`);
                break;
            }
            else {
                const paramName = token.val;
                const value = paramsArray ? paramsArray[paramsIndex] : paramsObject[paramName];
                /**
                 * Type = 1 means param is required
                 */
                if (token.type === 1) {
                    this.#ensureHasParamValue(pattern, paramName, value);
                }
                paramsIndex++;
                if (value !== undefined && value !== null) {
                    uriSegments.push(`${value}${token.end}`);
                }
            }
        }
        return `/${uriSegments.join('/')}`;
    }
    /**
     * Suffix the query string to the URL
     */
    #suffixQueryString(url, qs) {
        if (qs) {
            const queryString = this.#qsParser.stringify(qs);
            url = queryString ? `${url}?${queryString}` : url;
        }
        return url;
    }
    /**
     * Prefixes base URL to the uri string
     */
    #prefixBaseUrl(uri) {
        return this.#baseUrl ? `${this.#baseUrl}${uri}` : uri;
    }
    /**
     * Prefix a custom base URL to the final URI
     */
    prefixUrl(url) {
        this.#baseUrl = url;
        return this;
    }
    /**
     * Disable route lookup. Calling this method considers
     * the "identifier" as the route pattern
     */
    disableRouteLookup() {
        this.#shouldPerformLookup = false;
        return this;
    }
    /**
     * Append query string to the final URI
     */
    qs(queryString) {
        if (!queryString) {
            return this;
        }
        this.#qs = queryString;
        return this;
    }
    /**
     * Specify params to apply to the route pattern
     */
    params(params) {
        if (!params) {
            return this;
        }
        this.#params = params;
        return this;
    }
    /**
     * Generate URL for the given route identifier. The identifier can be the
     * route name, controller.method name or the route pattern
     * itself.
     */
    make(identifier) {
        let url;
        if (this.#shouldPerformLookup) {
            const route = this.#routeFinder.findOrFail(identifier);
            url = this.#processPattern(route.pattern);
        }
        else {
            url = this.#processPattern(identifier);
        }
        return this.#suffixQueryString(this.#prefixBaseUrl(url), this.#qs);
    }
    /**
     * Generate a signed URL for the given route identifier. The identifier can be the
     * route name, controller.method name or the route pattern
     * itself.
     */
    makeSigned(identifier, options) {
        let url;
        if (this.#shouldPerformLookup) {
            const route = this.#routeFinder.findOrFail(identifier);
            url = this.#processPattern(route.pattern);
        }
        else {
            url = this.#processPattern(identifier);
        }
        /*
         * Making the signature from the qualified url. We do not prefix the domain when
         * making signature, since it just makes the signature big.
         *
         * There might be a case, when someone wants to generate signature for the same route
         * on their 2 different domains, but we ignore that case for now and can consider
         * it later (when someone asks for it)
         */
        const signature = this.#encryption.verifier.sign(this.#suffixQueryString(url, this.#qs), options?.expiresIn, options?.purpose);
        const qs = Object.assign({}, this.#qs, { signature });
        return this.#suffixQueryString(this.#prefixBaseUrl(url), qs);
    }
}
